"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require("babel-runtime/helpers/slicedToArray");

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _isString = require("lodash/isString");

var _isString2 = _interopRequireDefault(_isString);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function PgConnectionArgOrderBy(builder, _ref) {
  var inflection = _ref.pgInflection;

  builder.hook("init", function (_, _ref2) {
    var newWithHooks = _ref2.newWithHooks,
        introspectionResultsByKind = _ref2.pgIntrospectionResultsByKind,
        GraphQLEnumType = _ref2.graphql.GraphQLEnumType;

    introspectionResultsByKind.class.map(function (table) {
      var tableTypeName = inflection.tableType(table.name, table.namespace && table.namespace.name);
      /* const TableOrderByType = */
      newWithHooks(GraphQLEnumType, {
        name: inflection.orderByType(tableTypeName),
        description: `Methods to use when ordering \`${tableTypeName}\`.`,
        values: {
          NATURAL: {
            value: {
              alias: null,
              specs: []
            }
          }
        }
      }, {
        pgIntrospection: table,
        isPgRowSortEnum: true
      });
    });
    return _;
  });
  builder.hook("GraphQLObjectType:fields:field:args", function (args, _ref3, _ref4) {
    var extend = _ref3.extend,
        getTypeByName = _ref3.getTypeByName,
        sql = _ref3.pgSql;
    var _ref4$scope = _ref4.scope,
        isPgConnectionField = _ref4$scope.isPgConnectionField,
        table = _ref4$scope.pgIntrospection,
        addArgDataGenerator = _ref4.addArgDataGenerator;

    if (!isPgConnectionField || !table || table.kind !== "class") {
      return args;
    }
    var tableTypeName = inflection.tableType(table.name, table.namespace && table.namespace.name);
    var TableOrderByType = getTypeByName(inflection.orderByType(tableTypeName));

    addArgDataGenerator(function connectionOrderBy(_ref5) {
      var orderBy = _ref5.orderBy;

      return {
        pgCursorPrefix: orderBy && orderBy.alias && sql.literal(orderBy && orderBy.alias),
        pgQuery: function pgQuery(queryBuilder) {
          if (orderBy != null) {
            var specs = orderBy.specs,
                unique = orderBy.unique;

            var orders = Array.isArray(specs[0]) || specs.length === 0 ? specs : [specs];
            orders.forEach(function (_ref6) {
              var _ref7 = (0, _slicedToArray3.default)(_ref6, 2),
                  col = _ref7[0],
                  ascending = _ref7[1];

              var expr = (0, _isString2.default)(col) ? sql.fragment`${queryBuilder.getTableAlias()}.${sql.identifier(col)}` : col;
              queryBuilder.orderBy(expr, ascending);
            });
            if (unique) {
              queryBuilder.setOrderIsUnique();
            }
          }
        }
      };
    });
    var defaultValueEnum = TableOrderByType.getValues().find(function (v) {
      return v.name === "PRIMARY_KEY_ASC";
    }) || TableOrderByType.getValues()[0];

    return extend(args, {
      orderBy: {
        description: `The method to use when ordering \`${tableTypeName}\`.`,
        type: TableOrderByType,
        defaultValue: defaultValueEnum && defaultValueEnum.value
      }
    });
  });
};
//# sourceMappingURL=PgConnectionArgOrderBy.js.map