"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var base64 = function base64(str) {
  return new Buffer(String(str)).toString("base64");
};

exports.default = function PgTablesPlugin(builder, _ref) {
  var inflection = _ref.pgInflection;

  builder.hook("init", function (_, _ref2) {
    var getNodeIdForTypeAndIdentifiers = _ref2.getNodeIdForTypeAndIdentifiers,
        nodeIdFieldName = _ref2.nodeIdFieldName,
        newWithHooks = _ref2.newWithHooks,
        sql = _ref2.pgSql,
        introspectionResultsByKind = _ref2.pgIntrospectionResultsByKind,
        getTypeByName = _ref2.getTypeByName,
        pgGqlTypeByTypeId = _ref2.pgGqlTypeByTypeId,
        pgGqlInputTypeByTypeId = _ref2.pgGqlInputTypeByTypeId,
        pg2GqlMapper = _ref2.pg2GqlMapper,
        gql2pg = _ref2.gql2pg,
        _ref2$graphql = _ref2.graphql,
        GraphQLObjectType = _ref2$graphql.GraphQLObjectType,
        GraphQLNonNull = _ref2$graphql.GraphQLNonNull,
        GraphQLID = _ref2$graphql.GraphQLID,
        GraphQLList = _ref2$graphql.GraphQLList,
        GraphQLInputObjectType = _ref2$graphql.GraphQLInputObjectType;

    var Cursor = getTypeByName("Cursor");
    introspectionResultsByKind.class.forEach(function (table) {
      var tablePgType = introspectionResultsByKind.type.filter(function (type) {
        return type.type === "c" && type.category === "C" && type.namespaceId === table.namespaceId && type.classId === table.id;
      })[0];
      if (!tablePgType) {
        throw new Error("Could not determine the type for this table");
      }
      if (pg2GqlMapper[tablePgType.id]) {
        // Already handled
        return;
      }
      /*
      table =
        { kind: 'class',
          id: '6484790',
          name: 'bundle',
          description: null,
          namespaceId: '6484381',
          typeId: '6484792',
          isSelectable: true,
          isInsertable: true,
          isUpdatable: true,
          isDeletable: true }
      */
      var schema = table.namespace;
      var primaryKeyConstraint = introspectionResultsByKind.constraint.filter(function (con) {
        return con.classId === table.id;
      }).filter(function (con) {
        return con.type === "p";
      })[0];
      var primaryKeys = primaryKeyConstraint && primaryKeyConstraint.keyAttributeNums.map(function (num) {
        return introspectionResultsByKind.attributeByClassIdAndNum[table.id][num];
      });
      var attributes = introspectionResultsByKind.attribute.filter(function (attr) {
        return attr.classId === table.id;
      }).sort(function (a1, a2) {
        return a1.num - a2.num;
      });
      var tableTypeName = inflection.tableType(table.name, schema && schema.name);
      var shouldHaveNodeId = nodeIdFieldName && table.isSelectable && primaryKeys && primaryKeys.length ? true : false;
      var TableType = newWithHooks(GraphQLObjectType, {
        description: table.description || tablePgType.description,
        name: tableTypeName,
        interfaces: function interfaces() {
          if (shouldHaveNodeId) {
            return [getTypeByName("Node")];
          } else {
            return [];
          }
        },
        fields: function fields(_ref3) {
          var addDataGeneratorForField = _ref3.addDataGeneratorForField,
              Self = _ref3.Self;

          var fields = {};
          if (shouldHaveNodeId) {
            // Enable nodeId interface
            addDataGeneratorForField(nodeIdFieldName, function () {
              return {
                pgQuery: function pgQuery(queryBuilder) {
                  queryBuilder.select(sql.fragment`json_build_array(${sql.join(primaryKeys.map(function (key) {
                    return sql.fragment`${queryBuilder.getTableAlias()}.${sql.identifier(key.name)}`;
                  }), ", ")})`, "__identifiers");
                }
              };
            });
            fields[nodeIdFieldName] = {
              description: "A globally unique identifier. Can be used in various places throughout the system to identify this single value.",
              type: new GraphQLNonNull(GraphQLID),
              resolve(data) {
                return data.__identifiers && getNodeIdForTypeAndIdentifiers.apply(undefined, [Self].concat((0, _toConsumableArray3.default)(data.__identifiers)));
              }
            };
          }
          return fields;
        }
      }, {
        pgIntrospection: table,
        isPgRowType: table.isSelectable,
        isPgCompoundType: !table.isSelectable
      });
      var pgInputFields = {};
      var TableInputType = newWithHooks(GraphQLInputObjectType, {
        description: `An input for mutations affecting \`${tableTypeName}\``,
        name: inflection.inputType(TableType)
      }, {
        pgIntrospection: table,
        isInputType: true,
        isPgRowType: table.isSelectable,
        isPgCompoundType: !table.isSelectable,
        pgAddSubfield(fieldName, attrName, pgType, spec) {
          pgInputFields[fieldName] = {
            name: attrName,
            type: pgType
          };
          return spec;
        }
      });

      pg2GqlMapper[tablePgType.id] = {
        map: function map(_) {
          return _;
        },
        unmap: function unmap(obj) {
          return sql.fragment`row(${sql.join(attributes.map(function (attr) {
            var fieldName = inflection.column(attr.name, table.name, table.namespace.name);
            var pgInputField = pgInputFields[fieldName];
            var v = obj[fieldName];
            if (pgInputField && v != null) {
              var type = pgInputField.type;

              return sql.fragment`${gql2pg(v, type)}::${sql.identifier(type.namespaceName, type.name)}`;
            } else {
              return sql.null;
            }
          }), ",")})::${sql.identifier(tablePgType.namespaceName, tablePgType.name)}`;
        }
      };

      if (table.isSelectable) {
        /* const TablePatchType = */
        newWithHooks(GraphQLInputObjectType, {
          description: `Represents an update to a \`${tableTypeName}\`. Fields that are set will be updated.`,
          name: inflection.patchType(TableType)
        }, {
          pgIntrospection: table,
          isPgRowType: table.isSelectable,
          isPgCompoundType: !table.isSelectable,
          isPgPatch: true,
          pgAddSubfield(fieldName, _attrName, _type, spec) {
            // We don't use this currently
            return spec;
          }
        });
      }
      var EdgeType = newWithHooks(GraphQLObjectType, {
        description: `A \`${tableTypeName}\` edge in the connection.`,
        name: inflection.edge(TableType.name),
        fields: function fields(_ref4) {
          var fieldWithHooks = _ref4.fieldWithHooks,
              recurseDataGeneratorsForField = _ref4.recurseDataGeneratorsForField;

          recurseDataGeneratorsForField("node");
          return {
            cursor: fieldWithHooks("cursor", function (_ref5) {
              var addDataGenerator = _ref5.addDataGenerator;

              addDataGenerator(function () {
                return {
                  usesCursor: [true]
                };
              });
              return {
                description: "A cursor for use in pagination.",
                type: Cursor,
                resolve(data) {
                  return data.__cursor && base64((0, _stringify2.default)(data.__cursor));
                }
              };
            }),
            node: {
              description: `The \`${tableTypeName}\` at the end of the edge.`,
              type: new GraphQLNonNull(TableType),
              resolve(data) {
                return data;
              }
            }
          };
        }
      }, {
        isEdgeType: true,
        isPgRowEdgeType: true,
        nodeType: TableType,
        pgIntrospection: table
      });
      var PageInfo = getTypeByName("PageInfo");
      /*const ConnectionType = */
      newWithHooks(GraphQLObjectType, {
        description: `A connection to a list of \`${tableTypeName}\` values.`,
        name: inflection.connection(TableType.name),
        fields: function fields(_ref6) {
          var recurseDataGeneratorsForField = _ref6.recurseDataGeneratorsForField;

          recurseDataGeneratorsForField("edges");
          recurseDataGeneratorsForField("nodes");
          recurseDataGeneratorsForField("pageInfo");
          return {
            nodes: {
              description: `A list of \`${tableTypeName}\` objects.`,
              type: new GraphQLNonNull(new GraphQLList(TableType)),
              resolve(data) {
                return data.data;
              }
            },
            edges: {
              description: `A list of edges which contains the \`${tableTypeName}\` and cursor to aid in pagination.`,
              type: new GraphQLNonNull(new GraphQLList(new GraphQLNonNull(EdgeType))),
              resolve(data) {
                return data.data;
              }
            },
            pageInfo: PageInfo && {
              description: "Information to aid in pagination.",
              type: new GraphQLNonNull(PageInfo),
              resolve(data) {
                return data;
              }
            }
          };
        }
      }, {
        isConnectionType: true,
        isPgRowConnectionType: true,
        edgeType: EdgeType,
        nodeType: TableType,
        pgIntrospection: table
      });
      pgGqlTypeByTypeId[tablePgType.id] = TableType;
      pgGqlInputTypeByTypeId[tablePgType.id] = TableInputType;
    });
    return _;
  });
};
//# sourceMappingURL=PgTablesPlugin.js.map