"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.postGraphQLClassicIdsInflection = exports.postGraphQLInflection = exports.defaultInflection = undefined;

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _pluralize = require("pluralize");

var _pluralize2 = _interopRequireDefault(_pluralize);

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const defaultInflection = exports.defaultInflection = {
  pluralize: _pluralize2.default,
  argument(name, index) {
    return (0, _utils.camelCase)(name || `arg${index}`);
  },
  orderByType(typeName) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`${(0, _pluralize2.default)(typeName)}-order-by`));
  },
  orderByEnum(name, ascending, _table, _schema) {
    return (0, _utils.constantCase)(`${name}_${ascending ? "asc" : "desc"}`);
  },
  domainType(name) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(name));
  },
  enumName(value) {
    return value;
  },
  enumType(name) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(name));
  },
  conditionType(typeName) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`${typeName}-condition`));
  },
  inputType(typeName) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`${typeName}-input`));
  },
  rangeBoundType(typeName) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`${typeName}-range-bound`));
  },
  rangeType(typeName) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`${typeName}-range`));
  },
  patchType(typeName) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`${typeName}-patch`));
  },
  patchField(itemName) {
    return (0, _utils.camelCase)(`${itemName}-patch`);
  },
  tableName(name, _schema) {
    return (0, _utils.camelCase)(_pluralize2.default.singular(name));
  },
  tableNode(name, _schema) {
    return (0, _utils.camelCase)(_pluralize2.default.singular(name));
  },
  allRows(name, schema) {
    return (0, _utils.camelCase)(`all-${this.pluralize(this.tableName(name, schema))}`);
  },
  functionName(name, _schema) {
    return (0, _utils.camelCase)(name);
  },
  functionPayloadType(name, _schema) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`${name}-payload`));
  },
  functionInputType(name, _schema) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`${name}-input`));
  },
  tableType(name, schema) {
    return (0, _utils.upperFirst)(this.tableName(name, schema));
  },
  column(name, _table, _schema) {
    return (0, _utils.camelCase)(name);
  },
  singleRelationByKeys(detailedKeys, table, schema) {
    return (0, _utils.camelCase)(`${this.tableName(table, schema)}-by-${detailedKeys.map(key => this.column(key.column, key.table, key.schema)).join("-and-")}`);
  },
  updateByKeys(detailedKeys, table, schema) {
    return (0, _utils.camelCase)(`update-${this.tableName(table, schema)}-by-${detailedKeys.map(key => this.column(key.column, key.table, key.schema)).join("-and-")}`);
  },
  deleteByKeys(detailedKeys, table, schema) {
    return (0, _utils.camelCase)(`delete-${this.tableName(table, schema)}-by-${detailedKeys.map(key => this.column(key.column, key.table, key.schema)).join("-and-")}`);
  },
  updateNode(name, _schema) {
    return (0, _utils.camelCase)(`update-${_pluralize2.default.singular(name)}`);
  },
  deleteNode(name, _schema) {
    return (0, _utils.camelCase)(`delete-${_pluralize2.default.singular(name)}`);
  },
  updateByKeysInputType(detailedKeys, name, _schema) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`update-${_pluralize2.default.singular(name)}-by-${detailedKeys.map(key => this.column(key.column, key.table, key.schema)).join("-and-")}-input`));
  },
  deleteByKeysInputType(detailedKeys, name, _schema) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`delete-${_pluralize2.default.singular(name)}-by-${detailedKeys.map(key => this.column(key.column, key.table, key.schema)).join("-and-")}-input`));
  },
  updateNodeInputType(name, _schema) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`update-${_pluralize2.default.singular(name)}-input`));
  },
  deleteNodeInputType(name, _schema) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`delete-${_pluralize2.default.singular(name)}-input`));
  },
  manyRelationByKeys(detailedKeys, table, schema) {
    return (0, _utils.camelCase)(`${this.pluralize(this.tableName(table, schema))}-by-${detailedKeys.map(key => this.column(key.column, key.table, key.schema)).join("-and-")}`);
  },
  edge(typeName) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`${(0, _pluralize2.default)(typeName)}-edge`));
  },
  edgeField(name, _schema) {
    return (0, _utils.camelCase)(`${_pluralize2.default.singular(name)}-edge`);
  },
  connection(typeName) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`${this.pluralize(typeName)}-connection`));
  },
  scalarFunctionConnection(procName, _procSchema) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`${procName}-connection`));
  },
  scalarFunctionEdge(procName, _procSchema) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`${procName}-edge`));
  },
  createField(name, _schema) {
    return (0, _utils.camelCase)(`create-${_pluralize2.default.singular(name)}`);
  },
  createInputType(name, _schema) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`create-${_pluralize2.default.singular(name)}-input`));
  },
  createPayloadType(name, _schema) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`create-${_pluralize2.default.singular(name)}-payload`));
  },
  updatePayloadType(name, _schema) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`update-${_pluralize2.default.singular(name)}-payload`));
  },
  deletePayloadType(name, _schema) {
    return (0, _utils.upperFirst)((0, _utils.camelCase)(`delete-${_pluralize2.default.singular(name)}-payload`));
  }
};

const postGraphQLInflection = exports.postGraphQLInflection = (0, _assign2.default)({}, defaultInflection, {
  enumName(value) {
    return (0, _utils.constantCase)(value);
  }
});

const postGraphQLClassicIdsInflection = exports.postGraphQLClassicIdsInflection = (0, _assign2.default)({}, postGraphQLInflection, {
  column(name, _table, _schema) {
    return name === "id" ? "rowId" : (0, _utils.camelCase)(name);
  }
});
//# sourceMappingURL=inflections.js.map